package net.minecraftforge.common.chunkio;

import net.minecraft.world.ChunkCoordIntPair;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.world.chunk.storage.AnvilChunkLoader;
import net.minecraft.world.gen.ChunkProviderServer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.world.ChunkDataEvent;

import java.io.IOException;
import java.util.concurrent.ConcurrentLinkedQueue;

class ChunkIOProvider implements Runnable
{
    private QueuedChunk chunkInfo;
    private Chunk chunk;
    private NBTTagCompound nbt;
    private ConcurrentLinkedQueue<Runnable> callbacks = new ConcurrentLinkedQueue<Runnable>();
    private boolean ran = false;

    ChunkIOProvider(QueuedChunk chunk)
    {
        this.chunkInfo = chunk;
    }

    public void addCallback(Runnable callback)
    {
        this.callbacks.add(callback);
    }
    public void removeCallback(Runnable callback)
    {
        this.callbacks.remove(callback);
    }

    @Override
    public void run() // async stuff
    {
        synchronized(this)
        {
            AnvilChunkLoader loader = chunkInfo.loader;
            Object[] data = null;
            try
            {
                data = loader.loadChunk__Async(chunkInfo.world, chunkInfo.x, chunkInfo.z);
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }

            if (data != null)
            {
                this.nbt   = (NBTTagCompound)data[1];
                this.chunk = (Chunk)data[0];
            }

            this.ran = true;
            this.notifyAll();
        }
    }

    // sync stuff
    public void syncCallback()
    {
        ChunkProviderServer provider = this.chunkInfo.provider;
        if (chunk == null)
        {
            // If the chunk loading failed just do it synchronously (may generate)
            provider.originalLoadChunk(this.chunkInfo.x, this.chunkInfo.z);
            return;
        }

        // Load Entities
        this.chunkInfo.loader.loadEntities(this.chunkInfo.world, this.nbt.func_74775_l("Level"), this.chunk);

        MinecraftForge.EVENT_BUS.post(new ChunkDataEvent.Load(this.chunk, this.nbt)); // Don't call ChunkDataEvent.Load async

        this.chunk.func_177432_b(provider.field_73251_h.func_82737_E());
        provider.field_73244_f.func_76163_a(ChunkCoordIntPair.func_77272_a(this.chunkInfo.x, this.chunkInfo.z), this.chunk);
        provider.field_73245_g.add(this.chunk);
        this.chunk.func_76631_c();

        if (provider.field_186029_c != null)
        {
            provider.field_186029_c.func_180514_a(this.chunk, this.chunkInfo.x, this.chunkInfo.z);
        }

        this.chunk.func_186030_a(provider, provider.field_186029_c);

        for (Runnable r : this.callbacks)
        {
            r.run();
        }

        this.callbacks.clear();
    }

    public Chunk getChunk()
    {
        return this.chunk;
    }

    public boolean runFinished()
    {
        return this.ran;
    }

    public boolean hasCallback()
    {
        return this.callbacks.size() > 0;
    }
}